"""All API endpoints in one place."""

import typing

import yarl

from kuro import types

__all__ = [
    "GACHA_RECORD",
    "GAME_ANNOUNCEMENTS",
    "GAME_ANNOUNCEMENT_DETAILS",
    "GAME_AUTO_LOGIN",
    "GAME_LOGIN",
    "GAME_TOKEN",
    "GAME_TOKEN_CHECK",
    "GAME_USER_INFO",
    "GET_SMS_CODE",
    "LAUNCHER_ANNOUNCEMENT_DETAILS",
    "LAUNCHER_ANNOUNCEMENT_LIST",
    "WEB_LOGIN",
    "InternationalRoute",
    "Route",
]


class Route:
    """Standard route."""

    url: yarl.URL

    def __init__(self, url: str) -> None:
        self.url = yarl.URL(url)

    def get_url(self) -> yarl.URL:
        """Get the route URL."""
        return self.url


class InternationalRoute:
    """Standard international route."""

    urls: typing.Mapping[types.Region, yarl.URL]

    def __init__(self, *, overseas: str, chinese: str) -> None:
        self.urls = {
            types.Region.OVERSEAS: yarl.URL(overseas),
            types.Region.CHINESE: yarl.URL(chinese),
        }

    def get_url(self, region: types.Region) -> yarl.URL:
        """Attempt to get a URL."""
        if not self.urls[region]:
            raise RuntimeError(f"URL does not support {region.name} region.")

        return self.urls[region]


GACHA_RECORD = Route("https://gmserver-api.aki-game2.net/gacha/record/query")

GET_SMS_CODE = Route("https://api.kurobbs.com/user/getSmsCodeForH5")

WEB_LOGIN = Route("https://api.kurobbs.com/user/loginForH5")

GAME_ANNOUNCEMENTS = InternationalRoute(
    overseas="https://aki-gm-resources-back.aki-game.net/gamenotice/G153/6eb2a235b30d05efd77bedb5cf60999e/notice.json",
    chinese="https://aki-gm-resources-back.aki-game.com/gamenotice/G152/76402e5b20be2c39f095a152090afddc/notice.json",
)

GAME_ANNOUNCEMENT_DETAILS = InternationalRoute(
    overseas="https://aki-gm-resources-back.aki-game.net/gamenotice/content/G153/6eb2a235b30d05efd77bedb5cf60999e/",
    chinese="https://aki-gm-resources-back.aki-game.com/gamenotice/content/G152/76402e5b20be2c39f095a152090afddc/",
)

LAUNCHER_ANNOUNCEMENT_LIST = InternationalRoute(
    overseas="https://prod-alicdn-gamestarter.kurogame.com/launcher/50004_obOHXFrFanqsaIEOmuKroCcbZkQRBC7c/G153/information/",
    chinese="https://prod-cn-alicdn-gamestarter.kurogame.com/launcher/10003_Y8xXrXk65DqFHEDgApn3cpK5lfczpFx5/G152/information/zh-Hans.json",
)

LAUNCHER_ANNOUNCEMENT_DETAILS = Route(
    "https://hw-media-cdn-mingchao.kurogame.com/akiwebsite/website2.0/json/G152/"
)

GAME_LOGIN = Route("https://sdkapi.kurogame-service.com/sdkcom/v2/login/emailPwd.lg")

GAME_TOKEN = Route("https://sdkapi.kurogame-service.com/sdkcom/v2/auth/getToken.lg")

GAME_TOKEN_CHECK = Route("https://sdkapi.kurogame-service.com/sdkcom/v2/heartbeat/tokenCheck.lg")

GAME_AUTO_LOGIN = Route("https://sdkapi.kurogame-service.com/sdkcom/v2/login/auto.lg")

GAME_USER_INFO = Route("https://gar-service.aki-game.net/UserRegion/GetUserInfo")

OAUTH_CODE = Route("https://sdkapi.kurogame-service.com/sdkcom/v2/user/oauth/code/generate.lg")

LAUNCHER_PLAYER_INFO = Route("https://pc-launcher-sdk-api.kurogame.net/game/queryPlayerInfo")

LAUNCHER_PLAYER_ROLE = Route("https://pc-launcher-sdk-api.kurogame.net/game/queryRole")



"""Auth component responsible for authentication."""

from __future__ import annotations

from typing import TYPE_CHECKING

from kuro import errors, types
from kuro.client.components.auth import subclients
from kuro.utility import geetest

if TYPE_CHECKING:
    from kuro import models

__all__ = ["AuthClient"]


class AuthClient(subclients.GameAuthClient, subclients.WebAuthClient):
    """Auth client."""

    async def demo_web_login(self, number: str, *, port: int = 5000) -> models.LoginResult:
        """Login with a phone number.

        ### Args:
            number: Chinese phone number. With or without regional prefix (`+86`).
            port: Port to use for geetest server.

        ### Returns:
            Login result.
        """
        success = await self.send_sms_code(number)
        if not success:
            mmt_result = await geetest.server.solve_geetest(
                captcha_id=types.CaptchaId.KUROBBS, lang=self.lang, port=port
            )
            await self.send_sms_code(number, mmt_result=mmt_result)

        code = input("Enter OTP code: ")  # noqa: ASYNC250
        return await self.web_login(number, code)

    async def demo_game_login(
        self, email: str, password: str, *, port: int = 5000
    ) -> models.GameLoginResult:
        """Login to game account with email and password.

        ### Args:
            email: Account email.
            password: Account password.

        ### Returns:
            Login result.
        """
        try:
            login_result = await self.game_login(email, password)
        except errors.GeetestTriggeredError:
            mmt_result = await geetest.server.solve_geetest(
                captcha_id=types.CaptchaId.GAME, lang=self.lang, port=port
            )
            login_result = await self.game_login(email, password, mmt_result=mmt_result)

        return login_result



"""Component for game announcements."""

from __future__ import annotations

from kuro import models
from kuro.client import routes
from kuro.client.components import base

__all__ = ["GameAnnouncementClient"]


class GameAnnouncementClient(base.BaseClient):
    """Announcement Client."""

    async def get_game_announcements(self) -> models.GameAnnouncementResult:
        """Get game announcements.

        ### Returns:
            Game Announcements.
        """
        rsp = await self.request(routes.GAME_ANNOUNCEMENTS.get_url(self.region))

        return models.GameAnnouncementResult(**rsp)

    async def get_game_announcement_details(
        self, announcement_id: str
    ) -> models.GameAnnouncementDetails:
        """Get game announcement details.

        Can give either id or url from game_announcements

        ### Example:
        ```python
        announcement_list = await client.get_game_announcements()

        for announcement in announcement_list.game:
            details = await client.get_game_announcement_details(announcement.id)
            print(details.title)
        ```

        ### Args:
            id: ID of the announcement to get details for

        ### Returns:
            Game announcement details.
        """
        rsp = await self.request(
            routes.GAME_ANNOUNCEMENT_DETAILS.get_url(self.region)
            / f"{announcement_id}/{self.lang.value}.json"
        )

        return models.GameAnnouncementDetails(**rsp)



"""Game announcement models."""

import typing

import pydantic

from kuro.models import base

__all__ = [
    "GameAnnouncementDetails",
    "GameAnnouncementRecord",
    "GameAnnouncementResult",
    "GameLocalizedAssetsMap",
    "GameLocalizedTextMap",
]


class GameAnnouncementDetails(base.APIModel):
    """Announcement Details."""

    announcement_id: int = pydantic.Field(alias="noticeId")
    """Announcement ID."""
    html_content: str = pydantic.Field(alias="textContent")
    """HTML content."""
    title: str = pydantic.Field(alias="textTitle")
    """Announcement title."""
    banner: str
    """Banner image."""


class GameLocalizedAssetsMap(base.APIModel):
    """Languages for asset fields."""

    zh_hans: typing.Sequence[str] | None = pydantic.Field(alias="zh-Hans", default=None)
    """Chinese Simplified."""
    zh_hant: typing.Sequence[str] | None = pydantic.Field(alias="zh-Hant", default=None)
    """Chinese Traditional."""
    en: typing.Sequence[str] | None = None
    """English."""
    ja: typing.Sequence[str] | None = None
    """Japanese."""
    ko: typing.Sequence[str] | None = None
    """Korean."""
    fr: typing.Sequence[str] | None = None
    """French."""
    de: typing.Sequence[str] | None = None
    """German."""
    es: typing.Sequence[str] | None = None
    """Spanish."""


class GameLocalizedTextMap(base.APIModel):
    """Languages for text fields."""

    zh_hans: str | None = pydantic.Field(alias="zh-Hans", default=None)
    """Chinese Simplified."""
    zh_hant: str | None = pydantic.Field(alias="zh-Hant", default=None)
    """Chinese Traditional."""
    en: str | None = None
    """English."""
    ja: str | None = None
    """Japanese."""
    ko: str | None = None
    """Korean."""
    fr: str | None = None
    """French."""
    de: str | None = None
    """German."""
    es: str | None = None
    """Spanish."""


class GameAnnouncementRecord(base.APIModel):
    """Game announcement record model.

    `red`, `channel`, `platform`, `whitelist` are used to determine
    whether the announcement should be shown on webview.

    See [this message](https://discord.com/channels/1242463166592585871/1242652304168190026/1250356292946624612)
    for more details.
    """

    details_url: str = pydantic.Field(alias="contentPrefix")
    """Url for announcement details."""
    red: int
    """Should announcement be processed."""  # needs more checking
    permanent: int
    """Permanent announcement."""
    id: str
    """Announcement id."""
    start_time_ms: int
    """Unix start time stamp."""
    end_time_ms: int
    """Unix end time stamp."""
    platform: typing.Sequence[int]
    """Platform for announcement to show on."""
    channel: typing.Sequence[typing.Any]
    """If channel doesn't exist then show."""
    white_list: typing.Sequence[typing.Any]
    """If whitelist is empty then show if permanent."""
    title: GameLocalizedTextMap = pydantic.Field(alias="tabTitle")
    """Title of announcement."""
    banner: GameLocalizedAssetsMap = pydantic.Field(alias="tabBanner")
    """Banner of announcement."""

    @pydantic.field_validator("details_url", mode="before")
    @classmethod
    def __cast_details(cls, v: typing.Sequence[str]) -> str:  # type: ignore
        return "".join(str(item) for item in v)


class GameAnnouncementResult(base.APIModel):
    """Full Announcement page model."""

    game: typing.Sequence[GameAnnouncementRecord]
    """Game Announcement."""
    event: typing.Sequence[GameAnnouncementRecord] = pydantic.Field(alias="activity")
    """Event Announcement"""



"""Simple geetest server implementation for solving captcha."""

from __future__ import annotations

import asyncio
import typing
import webbrowser

import aiohttp
from aiohttp import web

from kuro import models, types
from kuro.utility import geetest

__all__ = ["PAGE", "launch_server", "solve_geetest"]

PAGE = """
<!DOCTYPE html>
<head>
  <meta name="referrer" content="no-referrer"/>
</head>
<html>
  <body></body>
  <script src="./gt.js"></script>
  <script>
      window.initGeetest4(
        {
          captchaId: "%s",
          product: "bind",
          language: "%s",
        },
        (captcha) => {
          captcha.onReady(() => {
            captcha.showCaptcha();
          });
          captcha.onSuccess(() => {
            fetch("/send-data", {
              method: "POST",
              body: JSON.stringify(captcha.getValidate()),
            }).then(() => window.close());
            document.body.innerHTML = "You may now close this window.";
          });
        }
      )
  </script>
</html>
"""

GT_URL = "https://static.geetest.com/v4/gt4.js"


async def launch_server(*, captcha_id: str, lang: str, port: int = 5000) -> models.MMTResult:
    """Create and run a web server to solve captcha."""
    routes = web.RouteTableDef()
    future: asyncio.Future[typing.Any] = asyncio.Future()

    @routes.get("/")
    async def index(_: web.Request) -> web.StreamResponse:  # noqa: RUF029
        body = PAGE % (str(captcha_id), lang or "en")
        return web.Response(body=body, content_type="text/html")

    @routes.get("/gt.js")
    async def gt(_: web.Request) -> web.StreamResponse:
        async with aiohttp.ClientSession() as session:
            r = await session.get(GT_URL)
            content = await r.read()

        return web.Response(body=content, content_type="text/javascript")

    @routes.post("/send-data")
    async def send_data_endpoint(request: web.Request) -> web.Response:
        result = await request.json()
        future.set_result(models.MMTResult(**result))
        return web.Response(status=204)

    app = web.Application()
    app.add_routes(routes)

    runner = web.AppRunner(app)
    await runner.setup()

    site = web.TCPSite(runner, host="localhost", port=port)
    print(f"Opening http://localhost:{port} in browser...")  # noqa
    webbrowser.open_new_tab(f"http://localhost:{port}")

    await site.start()

    try:
        data = await future
    finally:
        await asyncio.sleep(0.3)
        await runner.shutdown()
        await runner.cleanup()

    return data


async def solve_geetest(
    *, captcha_id: types.CaptchaId | str, lang: types.Lang, port: int = 5000
) -> models.MMTResult:
    """Start a web server and manually solve geetest captcha."""
    geetest_lang = geetest.lang_to_geetest_lang(lang)
    captcha_id_str = captcha_id.value if isinstance(captcha_id, types.CaptchaId) else captcha_id
    return await launch_server(captcha_id=captcha_id_str, lang=geetest_lang, port=port)



"""Launcher announcement models."""

import re
import typing

import pydantic

from kuro import types
from kuro.models import base

__all__ = [
    "LauncherAnnouncement",
    "LauncherAnnouncementDetails",
    "LauncherAnnouncementGroup",
    "LauncherAnnouncementInner",
    "LauncherAnnouncementList",
    "LauncherSlideshowItem",
]


class LauncherAnnouncement(base.APIModel):
    """Launcher announcement item."""

    content: str
    """Launcher box content."""
    jump_url: str
    """Link to details."""
    time: str
    """Date in days of release."""

    @property
    def id(self) -> int:
        """Announcement Id."""
        parts = self.jump_url.split("/")
        post_id = parts[-1] if len(parts) > 0 else None
        assert post_id is not None, "Failed to extract ID from jump URL"
        return int(post_id)

    @property
    def lang(self) -> types.Lang:
        """Announcement language."""
        m = re.search(r"/(\w+)/main/news", self.jump_url)
        assert m is not None, "Failed to extract url from jump URL"
        return types.Lang(m.group(1))


class LauncherAnnouncementGroup(base.APIModel):
    """Launcher announcement group."""

    contents: typing.Sequence[LauncherAnnouncement]
    """List of announcements."""
    title: str
    """Group title."""


class LauncherAnnouncementInner(base.APIModel):
    """Launcher announcement inner group."""

    activity: LauncherAnnouncementGroup
    """Activity section."""
    description: str = pydantic.Field(alias="desc")
    """Description."""
    news: LauncherAnnouncementGroup
    """News section."""
    notice: LauncherAnnouncementGroup
    """Notice section."""


class LauncherSlideshowItem(base.APIModel):
    """Launcher slideshow item."""

    jump_url: str
    """Slideshow."""
    md5: str
    """md5 hash."""
    url: str
    """Slideshow picture url."""


class LauncherAnnouncementList(base.APIModel):
    """Launcher announcement list."""

    guidance: LauncherAnnouncementInner
    """Launcher Announcement Inner grouping."""
    slideshow: typing.Sequence[LauncherSlideshowItem]
    """Slideshow list."""


class LauncherAnnouncementDetails(base.APIModel):
    """Launcher announcement content."""

    article_content: str
    """html response"""
    article_id: int
    """Article id"""
    article_title: str
    """Article title"""
    article_type: int
    """Article type"""
    article_type_name: str
    """Article type name"""
    game_id: str
    """Game id"""
    start_time: str
    """ISO 8601 timestamp for when not to start showing article"""
    end_time: str | None = None
    """ISO 8601 timestamp for when not to show article"""

    @property
    def game(self) -> types.Game:
        """Game type."""
        return types.Game(self.game_id)



"""Crypto-related functions."""

import base64
import hashlib
import typing
import uuid


def encode_md5_parameter(m: dict[str, typing.Any], app_key: str) -> str:
    """Encode MD5 parameter."""
    builder: typing.Sequence[typing.Any] = []

    for key in sorted(m.keys()):
        if key not in {"sign", "market"} and m[key] is not None:
            builder.append(f"{key}={m[key]}&")

    builder.append(app_key)
    return md5_code("".join(builder))


def md5_code(code: str) -> str:
    """MD5 code."""
    b = encode_hex_md5(code).lower().encode()
    if len(b) >= 23:
        b = bytearray(b)
        b[1], b[13] = b[13], b[1]
        b[5], b[17] = b[17], b[5]
        b[7], b[23] = b[23], b[7]

    return b.decode()


def encode_password(password: str) -> str:
    """Encode password."""
    if not password:
        return ""
    pass_encoded = base64.b64encode(password.encode()).decode()
    p = list(pass_encoded)
    shuffle(p, 0)
    shuffle(p, 1)
    return "".join(p)


def shuffle(data: list[str], start_index: int) -> typing.Sequence[str]:
    """Shuffle data."""
    for i in range(start_index, len(data), 4):
        if i + 2 < len(data):
            data[i], data[i + 2] = data[i + 2], data[i]
        if i + 6 >= len(data):
            break
    return data


def encode_hex_md5(s: str) -> str:
    """Encode hex MD5."""
    try:
        return hashlib.md5(s.encode()).hexdigest()
    except Exception:
        return s


def generate_uuid() -> str:
    """Generate a UUID."""
    return str(uuid.uuid4())


def generate_uuid_uppercase() -> str:
    """Generate a UUID."""
    return str(uuid.uuid4()).upper()



"""Announcements Client."""

from kuro.client.components.announcements import subclients


class AnnouncementsClient(subclients.GameAnnouncementClient, subclients.LauncherAnnouncementClient):
    """Announcements client."""



"""Base client."""

from __future__ import annotations

import abc
import http.cookies
import json
import logging
import typing

import aiohttp
import aiohttp.typedefs
import yarl

if typing.TYPE_CHECKING:
    import aiocache

from kuro import types

CookieOrHeader = typing.Union[
    "http.cookies.BaseCookie[typing.Any]", typing.Mapping[typing.Any, typing.Any], str
]


def parse_cookie(cookie: CookieOrHeader | None) -> dict[str, str]:
    """Parse a cookie or header into a cookie mapping."""
    if cookie is None:
        return {}

    if isinstance(cookie, str):
        cookie = http.cookies.SimpleCookie(cookie)

    return {
        str(k): v.value if isinstance(v, http.cookies.Morsel) else str(v) for k, v in cookie.items()
    }


class BaseClient(abc.ABC):
    """Base client."""

    logger: logging.Logger = logging.getLogger(__name__)
    """Logger for the client."""

    _cookies: dict[str, str]
    """Cookies used for authentication."""

    def __init__(
        self,
        cookies: CookieOrHeader | None = None,
        *,
        lang: types.Lang = types.Lang.ENGLISH,
        region: types.Region = types.Region.OVERSEAS,
        cache: aiocache.Cache | None = None,
        debug: bool = False,
    ) -> None:
        self.cookies = parse_cookie(cookies) if cookies else {}
        self.lang = lang
        """Language to use for the API."""
        self.region = region
        """Region to use for the API."""
        self.cache = cache
        """Cache to store responses."""
        self.debug = debug
        """Whether to log debug information."""

    def _request_hook(
        self,
        method: str,
        url: aiohttp.typedefs.StrOrURL,
        *,
        params: typing.Mapping[str, typing.Any] | None = None,
        data: typing.Any = None,
        **_: typing.Any,
    ) -> None:
        """Perform an action before a request.

        Debug logging by default.
        """
        url = yarl.URL(url)
        if params:
            params = dict(params.items())
            url = url.update_query(params)

        if data:
            self.logger.debug("%s %s\n%s", method, url, json.dumps(data, separators=(",", ":")))
        else:
            self.logger.debug("%s %s", method, url)

    async def request(
        self,
        url: aiohttp.typedefs.StrOrURL,
        *,
        method: str | None = None,
        params: typing.Mapping[str, typing.Any] | None = None,
        data: typing.Any = None,
        headers: aiohttp.typedefs.LooseHeaders | None = None,
        use_cache: bool = True,
        **kwargs: typing.Any,
    ) -> typing.Mapping[str, typing.Any]:
        """Make a request to the API. All requests the library makes go through this method."""
        if method is None:
            method = "POST" if data else "GET"

        self._request_hook(method, url, params=params, data=data, headers=headers, **kwargs)
        key = self._gen_cache_key(url, method, params, data, headers, kwargs)

        if use_cache and self.cache and (cached := await self.cache.get(key)):
            return cached

        async with (
            aiohttp.ClientSession() as session,
            session.request(
                method,
                url,
                params=params,
                data=data,
                headers=headers,
                cookies=self.cookies,
                **kwargs,
            ) as response,
        ):
            data = await response.json()
            if use_cache and self.cache:
                await self.cache.set(key, data)

        return data

    def _gen_cache_key(
        self,
        url: aiohttp.typedefs.StrOrURL,
        method: str | None = None,
        params: typing.Mapping[str, typing.Any] | None = None,
        data: typing.Any = None,
        headers: aiohttp.typedefs.LooseHeaders | None = None,
        kwargs: typing.Any = None,
    ) -> str:
        return f"{url}_{method}_{params}_{data}_{headers}_{self.cookies}_{kwargs}"

    @property
    def cookies(self) -> typing.MutableMapping[str, str]:
        """Cookies used for authentication."""
        return self._cookies

    @cookies.setter
    def cookies(self, cookies: CookieOrHeader | None) -> None:
        if not cookies:
            self._cookies = {}
            return

        self._cookies = parse_cookie(cookies)

    def set_cookies(
        self,
        cookies: CookieOrHeader | None = None,
        **kwargs: typing.Any,
    ) -> typing.MutableMapping[str, str]:
        """Parse and set cookies."""
        if not bool(cookies) ^ bool(kwargs):
            msg = "Cannot use both positional and keyword arguments at once"
            raise TypeError(msg)

        self.cookies = parse_cookie(cookies or kwargs)
        return self.cookies

    @property
    def debug(self) -> bool:
        """Whether the debug mode is enabled."""
        return logging.getLogger("kuro").level == logging.DEBUG

    @debug.setter
    def debug(self, debug: bool) -> None:
        logging.basicConfig()
        level = logging.DEBUG if debug else logging.NOTSET
        logging.getLogger("kuro").setLevel(level)



"""Decorators for kuro clients."""

import functools
import typing

if typing.TYPE_CHECKING:
    from kuro import types

T = typing.TypeVar("T")
CallableT = typing.TypeVar("CallableT", bound="typing.Callable[..., object]")
AsyncCallableT = typing.TypeVar(
    "AsyncCallableT", bound="typing.Callable[..., typing.Awaitable[object]]"
)


def region_specific(region: "types.Region") -> typing.Callable[[AsyncCallableT], AsyncCallableT]:
    """Prevent function to be ran with unsupported regions."""

    def decorator(func: AsyncCallableT) -> AsyncCallableT:
        @functools.wraps(func)
        async def wrapper(self: typing.Any, *args: typing.Any, **kwargs: typing.Any) -> typing.Any:
            if not hasattr(self, "region"):
                raise TypeError("Cannot use @region_specific on a plain function.")
            if region != self.region:
                raise RuntimeError(
                    "The method can only be used with client region set to " + region
                )

            return await func(self, *args, **kwargs)

        return typing.cast("AsyncCallableT", wrapper)

    return decorator



"""Main client for requesting API endpoints."""

from kuro.client.components import announcements, auth, gacha, game

__all__ = ["Client"]


class Client(
    auth.AuthClient,
    gacha.GachaClient,
    game.GameClient,
    announcements.AnnouncementsClient,
):
    """Main client for requesting API endpoints."""



"""Gacha utility."""

from urllib.parse import parse_qs, urlparse

from kuro import models, types


def parse_gacha_url(url: str) -> models.ParsedGachaParams:
    """Parse gacha URL.

    Use the function to parse gacha url into a mapping that can be
    ued in `client.get_gacha_records` method:

    ```python
    url = "..."
    parsed = parse_gacha_url(url)
    records = await client.get_gacha_records(**parsed)
    ```

    Args:
        url: URL to parse.

    Returns:
        Parsed mapping that can be used in `client.get_gacha_records` method.
    """
    parsed_url = urlparse(url.replace("#", ""))
    query_params = parse_qs(parsed_url.query)
    return {
        "player_id": int(query_params["player_id"][0]),
        "record_id": query_params["record_id"][0],
        "server": types.WuWaServer(query_params["svr_id"][0]),
        "banner": types.WuWaBanner(int(query_params["gacha_type"][0])),
        "lang": types.Lang(query_params["lang"][0]),
    }



"""Gacha models."""

import datetime  # noqa: TC003
import enum
import typing

import pydantic

from kuro import constants, types
from kuro.models import assets, base

__all__ = ["GachaCharacter", "GachaItemType", "GachaRecord", "GachaWeapon", "ParsedGachaParams"]


class GachaItemType(enum.IntEnum):
    """Gacha item type."""

    RESONATOR = 1
    """Resonator."""
    WEAPON = 2
    """Weapon."""


class GachaRecord(base.APIModel):
    """Gacha record model."""

    banner: types.WuWaBanner
    """Banner type."""
    resource_id: int
    """Item ID."""
    rarity: int = pydantic.Field(alias="qualityLevel")
    """Item rarity."""
    type_name: str = pydantic.Field(alias="resourceType")
    """Localized item type name."""
    name: str
    """Name of the record item."""
    count: int
    """Count of the record item."""
    time: datetime.datetime
    """Time of the record."""

    @property
    def type(self) -> GachaItemType:
        """Item type."""
        return GachaItemType.WEAPON if self.resource_id >= 100000 else GachaItemType.RESONATOR


class GachaCharacter(GachaRecord):
    """Gacha character model."""

    @property
    def icon(self) -> assets.IconRoleHead:
        """Character icon."""
        return assets.IconRoleHead(
            url_150=(
                f"{constants.WUTHERY_CDN}/p/GameData/IDFiedResources/"
                f"Common/Image/IconRoleHead150/{self.resource_id}.png"
            ),
            url_175=(
                f"{constants.WUTHERY_CDN}/p/GameData/IDFiedResources/"
                f"Common/Image/IconRoleHead175/{self.resource_id}.png"
            ),
            url_256=(
                f"{constants.WUTHERY_CDN}/p/GameData/IDFiedResources/"
                f"Common/Image/IconRoleHead256/{self.resource_id}.png"
            ),
        )

    @property
    def icon_circle_url(self) -> str:
        """Character circle icon."""
        return (
            f"{constants.WUTHERY_CDN}/p/GameData/IDFiedResources/"
            f"Common/Image/IconRoleHeadCircle256/{self.resource_id}.png"
        )

    @property
    def pile_icon_url(self) -> str:
        """Character pile icon."""
        return (
            f"{constants.WUTHERY_CDN}/p/GameData/IDFiedResources/"
            f"Common/Image/IconRolePile/{self.resource_id}.png"
        )


class GachaWeapon(GachaRecord):
    """Gacha weapon model."""

    @property
    def icon(self) -> assets.IconWeapon:
        """Weapon icon."""
        return assets.IconWeapon(
            url_80=(
                f"{constants.WUTHERY_CDN}/p/GameData/IDFiedResources/"
                f"Common/Image/IconWeapon80/{self.resource_id}.png"
            ),
            url_160=(
                f"{constants.WUTHERY_CDN}/p/GameData/IDFiedResources/"
                f"Common/Image/IconWeapon160/{self.resource_id}.png"
            ),
            url_256=(
                f"{constants.WUTHERY_CDN}/p/GameData/IDFiedResources/"
                f"Common/Image/IconWeapon/{self.resource_id}.png"
            ),
        )


class ParsedGachaParams(typing.TypedDict):
    """Parsed gacha parameters."""

    player_id: int
    """Player ID."""
    record_id: str
    """Record ID (Unique to each user)."""
    banner: types.WuWaBanner
    """Banner type."""
    server: types.WuWaServer
    """Game server."""
    lang: types.Lang
    """Language."""



"""Game component for fetching game data."""

import json

from kuro import errors, models, types
from kuro.client import decorators, routes
from kuro.client.components import base


class GameClient(base.BaseClient):
    """Game client."""

    @decorators.region_specific(types.Region.OVERSEAS)
    async def get_game_user(
        self, user_id: int, token: str, area: str, username: str, *, login_type: int = 1
    ) -> models.GameUser:
        """Get game user info.

        ### Args:
            user_id: User ID.
            token: Access (game) token.
            area: Area.
            username: Username.
            login_type: Login type. Default is 1.

        ### Returns:
            Game info.
        """
        params: dict[str, str | int] = {
            "loginType": login_type,
            "userId": user_id,
            "token": token,
            "area": area,
            "userName": username,
        }
        rsp = await self.request(routes.GAME_USER_INFO.get_url().with_query(params))

        if rsp["Code"] not in {0, 49}:
            errors.raise_from_data(rsp)

        return models.GameUser(**rsp)

    @decorators.region_specific(types.Region.OVERSEAS)
    async def get_player_info(
        self, oauth_code: str, *, max_attempts: int = 3
    ) -> dict[str, models.GamePlayerInfo]:
        """Get player info.

        ### Args:
            oauth_code: OAuth code obtained from generate_oauth_code.
            max_attempts: Maximum number of attempts to get player info if failed. Default is 3.

        ### Returns:
            Regions mapped to player info. Example region is "Europe".
        """
        data = {"oauthCode": oauth_code}
        rsp = await self.request(routes.LAUNCHER_PLAYER_INFO.get_url(), method="POST", json=data)

        if max_attempts > 0 and rsp["code"] == 1005:
            return await self.get_player_info(oauth_code, max_attempts=max_attempts - 1)

        if rsp["code"] != 0:
            errors.raise_from_data(rsp)

        return {
            region: models.GamePlayerInfo(**json.loads(info))
            for region, info in rsp["data"].items()
        }

    @decorators.region_specific(types.Region.OVERSEAS)
    async def get_player_role(
        self,
        oauth_code: str,
        player_id: int,
        region: str,
        *,
        max_attempts: int = 3,
    ) -> models.RoleInfo:
        """Get player role info.

        ### Args:
            oauth_code: OAuth code obtained from generate_oauth_code.
            player_id: Player UID.
            region: Player region.
            max_attempts: Maximum number of attempts to get player role if failed. Default is 3.

        ### Returns:
            Player role info.
        """
        data: dict[str, int | str] = {
            "oauthCode": oauth_code,
            "playerId": player_id,
            "region": region,
        }
        rsp = await self.request(routes.LAUNCHER_PLAYER_ROLE.get_url(), method="POST", json=data)

        if max_attempts > 0 and rsp["code"] == 1005:
            return await self.get_player_role(
                oauth_code, player_id, region, max_attempts=max_attempts - 1
            )

        if rsp["code"] != 0:
            errors.raise_from_data(rsp)

        json_data = json.loads(rsp["data"][region])
        return models.RoleInfo(**json_data)



"""Base models for API responses."""

import pydantic
from pydantic import alias_generators

__all__ = ["APIModel"]


class APIModel(pydantic.BaseModel):
    """Base model for API responses."""

    model_config = pydantic.ConfigDict(
        alias_generator=alias_generators.to_camel, populate_by_name=True, from_attributes=True
    )



"""Component used to fetch gacha records."""

import typing

from kuro import errors, models, types
from kuro.client import decorators, routes
from kuro.client.components import base

__all__ = ["GachaClient"]


class GachaClient(base.BaseClient):
    """Gacha client."""

    @decorators.region_specific(types.Region.OVERSEAS)
    async def get_gacha_record(
        self,
        player_id: int,
        record_id: str,
        *,
        banner: types.WuWaBanner,
        server: types.WuWaServer,
        lang: types.Lang | None = None,
    ) -> typing.Sequence[models.GachaWeapon | models.GachaCharacter]:
        """Get gacha records.

        ### Example:
        ```python
        url = "..."
        parsed = parse_gacha_url(url)
        result = await client.get_gacha_record(**parsed)
        ```

        ### Args:
            player_id: Player ID.
            record_id: Record ID (unique for each player).
            banner: Banner you want to get record for.
            server: Game server.
            lang: Language.

        ### Returns:
            Sequence of gacha records.
        """
        body: dict[str, typing.Any] = {
            "playerId": str(player_id),
            "languageCode": lang.value if lang else self.lang.value,
            "cardPoolType": int(banner),
            "recordId": record_id,
            "serverId": server.value,
            "cardPoolId": "",
        }
        rsp = await self.request(routes.GACHA_RECORD.get_url(), method="POST", json=body)

        if rsp["code"] != 0:
            errors.raise_from_data(rsp)

        return [
            models.GachaWeapon(banner=banner, **record)
            if record["resourceId"] >= 100000
            else models.GachaCharacter(banner=banner, **record)
            for record in rsp["data"]
        ]



"""Component for launcher announcements."""

from __future__ import annotations

from kuro import models, types
from kuro.client import decorators, routes
from kuro.client.components import base

__all__ = ["LauncherAnnouncementClient"]


class LauncherAnnouncementClient(base.BaseClient):
    """Launcher Announcement Client."""

    async def get_launcher_announcements(self) -> models.LauncherAnnouncementList:
        """Get launcher announcement list.

        ### Example:
        ```python
        launcher_announcements = await client.get_launcher_announcements()
        print(launcher_announcements.guidance.activity.contents[0].content)
        ```

        ### Returns:
            Launcher official announcements list
        """
        url = routes.LAUNCHER_ANNOUNCEMENT_LIST.get_url(self.region)
        if self.region is types.Region.OVERSEAS:
            url /= f"{self.lang.value}.json"

        rsp = await self.request(url)

        return models.LauncherAnnouncementList(**rsp)

    @decorators.region_specific(types.Region.OVERSEAS)
    async def get_launcher_announcement_details(
        self, announcement_id: int
    ) -> models.LauncherAnnouncementDetails:
        """Get Launcher official announcement details.

        ### Example:
        ```python
            launcher_announcements = await client.get_launcher_announcements()

            id = launcher_announcements.guidance.activity.contents[0].id
            lang = launcher_announcements.guidance.activity.contents[0].lang

            details = await client.get_launcher_announcement_details(id, lang)
            print(details.article_title)
        ```

        ### Args:
            announcement_id: ID of the announcement to get details for

        ### Returns:
            Launcher announcement details
        """
        url = routes.LAUNCHER_ANNOUNCEMENT_DETAILS.get_url()
        rsp = await self.request(url / f"{self.lang.value}/article/{announcement_id}.json")

        return models.LauncherAnnouncementDetails(**rsp)



"""Constants for the lib."""

import typing

from kuro import types

__all__ = ["GEETEST_LANGS"]


GEETEST_LANGS: typing.Mapping[types.Lang, str] = {
    types.Lang.CHINESE_SIMPLIFIED: "zh-Hans",
    types.Lang.CHINESE_TRADITIONAL: "zh-Hant",
    types.Lang.ENGLISH: "en",
    types.Lang.JAPANESE: "ja",
    types.Lang.KOREAN: "ko",
    types.Lang.FRENCH: "fr",
    types.Lang.GERMAN: "de",
    types.Lang.SPANISH: "es",
}
"""Geetest languages."""

WUTHERY_CDN: str = "https://files.wuthery.com"
"""Wuthery CDN URL."""

APP_KEYS: typing.Mapping[types.Game, typing.Mapping[types.Region, str]] = {
    types.Game.WUWA: {
        types.Region.CHINESE: "",
        types.Region.OVERSEAS: "32gh5r0p35ullmxrzzwk40ly",
    }
}
"""App keys for game login."""



"""Game models."""

import datetime
import typing

import pydantic

from kuro.models import base


class GameAccount(base.APIModel):
    """User game account."""

    region: str = pydantic.Field(alias="Region")
    """Account region."""
    level: int = pydantic.Field(alias="Level")
    """Account level."""
    last_time_online: datetime.datetime = pydantic.Field(alias="LastOnlineTime")
    """Last time the user was online."""


class GameUser(base.APIModel):
    """Game user."""

    id: int = pydantic.Field(alias="UserId")
    """User ID."""
    sdk_login_code: int = pydantic.Field(alias="SdkLoginCode")
    """SDK login code."""
    accounts: typing.Sequence[GameAccount] = pydantic.Field(alias="UserInfos")
    """List of user game accounts."""
    recommended_region: str = pydantic.Field(alias="RecommendRegion")
    """Recommended region."""


class GamePlayerInfo(base.APIModel):
    """Game player info."""

    uid: int = pydantic.Field(alias="roleId")
    """User ID."""
    name: str = pydantic.Field(alias="roleName")
    """Player name."""
    level: int
    """Player level."""
    sex: int
    """Player sex."""
    avatar_id: int = pydantic.Field(alias="headPhoto")
    """Player avatar Id.

    See this file for more info: https://files.wuthery.com/GameData/ConfigDBParsed/PlayerHeadRe.json
    """


class BasicRoleInfo(base.APIModel):
    """Basic role info."""

    name: str = pydantic.Field(alias="Name")
    """Player name."""
    id: int = pydantic.Field(alias="Id")
    """Player ID."""
    create_time: datetime.datetime = pydantic.Field(alias="CreatTime")
    """Account creation time."""
    active_days: int = pydantic.Field(alias="ActiveDays")
    """Number of active days."""
    level: int = pydantic.Field(alias="Level")
    """Player level."""
    world_level: int = pydantic.Field(alias="WorldLevel")
    """World level."""
    character_count: int = pydantic.Field(alias="RoleNum")
    """Number of characters."""
    sonance_cascet_count: int = pydantic.Field(alias="SoundBox")
    """Number of sonance caskets."""
    waveplates: int = pydantic.Field(alias="Energy")
    """Current waveplates count."""
    max_waveplates: int = pydantic.Field(alias="MaxEnergy")
    """Maximum waveplates count."""
    refined_waveplates: int = pydantic.Field(alias="StoreEnergy")
    """Refined waveplates count."""
    max_refined_waveplates: int = pydantic.Field(alias="MaxStoreEnergy")
    """Maximum refined waveplates count."""
    refined_waveplates_replenish_time: datetime.datetime = pydantic.Field(
        alias="StoreEnergyRecoverTime"
    )
    """Refined waveplates replenish time."""
    waveplates_replenish_time: datetime.datetime = pydantic.Field(alias="EnergyRecoverTime")
    """Waveplates replenish time."""
    activity_points: int = pydantic.Field(alias="Liveness")
    """Current activity points."""
    max_activity_points: int = pydantic.Field(alias="LivenessMaxCount")
    """Maximum activity points."""
    activities_unlocked: bool = pydantic.Field(alias="LivenessUnlock")
    """Whether activities are unlocked."""
    chapter_id: int = pydantic.Field(alias="ChapterId")
    """Current chapter ID."""
    weekly_challenge: int = pydantic.Field(alias="WeeklyInstCount")
    """Weekly challenge count left."""
    chests: dict[str, int] = pydantic.Field(alias="Boxes")
    """Chests collected by the player. Key is a chest type (rarity)."""
    basic_chests: dict[str, int] = pydantic.Field(alias="BasicBoxes")
    """Basic chests collected."""
    tidal_heritages: dict[str, int] = pydantic.Field(alias="PhantomBoxes")
    """Tidal heritages collected. Key is a heritage type (rarity)."""
    birthday_month: int = pydantic.Field(alias="BirthMon")
    """Birthday month."""
    birthday_day: int = pydantic.Field(alias="BirthDay")
    """Birthday day."""


class BattlePassRoleInfo(base.APIModel):
    """Battle pass role info."""

    level: int = pydantic.Field(alias="Level")
    """Battle pass level."""
    weekly_xp: int = pydantic.Field(alias="WeekExp")
    """Weekly experience."""
    max_weekly_xp: int = pydantic.Field(alias="WeekMaxExp")
    """Maximum weekly experience."""
    is_unlocked: bool = pydantic.Field(alias="IsUnlock")
    """Whether the battle pass is unlocked."""
    is_opened: bool = pydantic.Field(alias="IsOpen")
    """Whether the battle pass is opened."""
    xp: int = pydantic.Field(alias="Exp")
    """Experience."""
    xp_limit: int = pydantic.Field(alias="ExpLimit")
    """Experience limit."""


class SkinInfo(base.APIModel):
    """Skin info."""

    id: int = pydantic.Field(alias="SkinId")
    """Skin ID."""
    quality: int = pydantic.Field(alias="Quality")
    """Skin quality."""


class BikeInfo(base.APIModel):
    """Bike info.

    For accounts that don't have bike unlocked, all int fields will be 0.
    """

    level: int = pydantic.Field(alias="Level")
    """Bike level."""
    xp: int = pydantic.Field(alias="Exp")
    """Bike experience."""
    next_xp: int = pydantic.Field(alias="NextExp")
    """Bike experience needed for next level."""
    skins: list[SkinInfo] = pydantic.Field(alias="Skins")
    """List of bike skins."""
    equipped_skin: SkinInfo = pydantic.Field(alias="EquipSkin")
    """Equipped bike skin."""


class MusicAlbum(base.APIModel):
    """Music album."""

    id: int = pydantic.Field(alias="Id")
    """Album ID."""
    count: int = pydantic.Field(alias="Count")
    """Number of songs collected in the album."""
    total_count: int = pydantic.Field(alias="TotalCount")
    """Total number of songs in the album."""


class MusicInfo(base.APIModel):
    """Music info."""

    albums: list[MusicAlbum] = pydantic.Field(alias="Albums")
    """List of music albums."""


class RoleInfo(base.APIModel):
    """Game role info."""

    basic: BasicRoleInfo = pydantic.Field(alias="Base")
    """Basic player info."""
    bike: BikeInfo = pydantic.Field(alias="MotorData")
    """Bike info."""
    music: MusicInfo = pydantic.Field(alias="MusicData")
    """Music info."""
    battle_pass: BattlePassRoleInfo = pydantic.Field(alias="BattlePass")
    """Battle pass info."""



"""Assets-related models."""

import pydantic

__all__ = ["IconRoleHead", "IconWeapon"]


class IconRoleHead(pydantic.BaseModel):
    """Role head icon model."""

    url_150: str
    """150x150 icon URL."""

    url_175: str
    """175x175 icon URL."""

    url_256: str
    """256x256 icon URL."""


class IconWeapon(pydantic.BaseModel):
    """Weapon icon model."""

    url_80: str
    """80x80 icon URL."""

    url_160: str
    """80x80 icon URL."""

    url_256: str
    """256x256 icon URL."""



"""Web auth models."""

import pydantic

from kuro.models import base

__all__ = ["LoginResult", "MMTResult"]


class LoginResult(base.APIModel):
    """Login result returned by `client.login`."""

    enable_child_mode: bool
    """Whether child mode is enabled."""
    gender: int
    """User gender."""
    signature: str
    """Signature."""
    avatar_url: str = pydantic.Field(alias="headUrl")
    """PNG avatar URL."""
    avatar_code: str = pydantic.Field(alias="headCode")
    """Avatar code."""
    user_name: str
    """User name."""
    user_id: str
    """User ID."""
    is_registered: int = pydantic.Field(alias="isRegister")
    """Whether the user is registered."""
    is_official: int
    """Whether the account is official."""
    status: int
    """User status."""
    token: str
    """Cookie token used to access API."""


class MMTResult(pydantic.BaseModel):
    """MMT result model."""

    captcha_id: str
    """CAPTCHA ID."""
    lot_number: str
    """Serial number."""
    pass_token: str
    """Token of successful verification."""
    gen_time: str
    """Timestamp."""
    captcha_output: str
    """Verification output data."""

    def get_game_dict(self) -> dict[str, str]:
        """Get dictionary for game geetest verification."""
        return {
            "geetestCaptchaOutput": self.captcha_output,
            "geetestGenTime": self.gen_time,
            "geetestLotNumber": self.lot_number,
            "geetestPassToken": self.pass_token,
        }



"""Lib exceptions and error handling logic."""

import typing

__all__ = (
    "GeetestTriggeredError",
    "KuroError",
)


class KuroError(Exception):
    """Base exception class."""

    retcode: int = 0
    """Kuro.py's custom error code."""
    msg: str = ""
    """Translated error message."""
    api_msg: str = ""
    """Error message returned by the API."""
    response: typing.Mapping[str, typing.Any]
    """Original API response."""

    def __init__(self, response: typing.Mapping[str, typing.Any]) -> None:
        self.response = response
        self.api_msg = response.get("msg", response.get("message", ""))

        self.retcode = self.retcode or RETCODES[self.api_msg]
        self.msg = self.msg or ERROR_TRANSLATIONS[self.retcode]

        super().__init__(self.msg)

    def __str__(self) -> str:
        return f"[{self.retcode}] {self.msg}"

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__} retcode={self.retcode} msg={self.msg!r} api_msg={self.api_msg!r} response={self.response}"


class GeetestTriggeredError(KuroError):
    """Geetest verification triggered error."""

    retcode = 41000
    msg = "Geetest verification triggered"


CUSTOM_ERRORS: typing.Mapping[int, type[KuroError]] = {
    41000: GeetestTriggeredError,
}
"""Mapping of kuro retcodes to custom error classes."""


RETCODES: typing.Mapping[str, int] = {
    "": 0,  # Unknown error
    "系统异常": 1,  # System error
    "请求游戏获取日志异常!": 2,  # Gacha history params expired
}
"""Kuro.py's custom error codes matched with API error messages."""


ERROR_TRANSLATIONS: typing.Mapping[int, str] = {
    0: "Unknown error occurred when requesting the API",
    1: "System error",
    2: "Gacha history params expired",
}
"""English error messages translated from Kuro.py's custom error codes."""


def raise_from_data(data: typing.Mapping[str, typing.Any]) -> None:
    """Raise an exception from API data."""
    code = data.get("code", data.get("codes"))
    if code in CUSTOM_ERRORS:
        raise CUSTOM_ERRORS[code](data)

    raise KuroError(data)



"""Geetest utility functions."""

from kuro import constants, types

__all__ = ["lang_to_geetest_lang"]


def lang_to_geetest_lang(lang: types.Lang) -> str:
    """Convert API language to geetest lang."""
    return constants.GEETEST_LANGS.get(lang, "en")



"""Game auth client."""

import typing

from kuro import constants, errors, models, types
from kuro.client import decorators, routes
from kuro.client.components import base
from kuro.utility import auth

__all__ = ["GameAuthClient"]


class GameAuthClient(base.BaseClient):
    """Game auth client."""

    @decorators.region_specific(types.Region.OVERSEAS)
    async def game_login(
        self,
        email: str,
        password: str,
        *,
        device_id: str | None = None,
        mmt_result: models.MMTResult | None = None,
    ) -> models.GameLoginResult:
        """Login with a username and password into game account.

        ### Args:
            email: Email (raw unencrypted string).
            password: Password (raw unencrypted string).
            device_id: Device ID (UUID4 uppercase). If not provided, a new UUID will be generated.
            mmt_result: MMT result from geetest.

        ### Returns:
            Login result.
        """
        data: dict[str, typing.Any] = {
            "__e__": 1,
            "email": email,
            "client_id": "7rxmydkibzzsf12om5asjnoo",  # KR_PRODUCT_KEY in kr_sdk_config.json
            "deviceNum": device_id or auth.generate_uuid_uppercase(),
            "password": auth.encode_password(password),
            "platform": "PC",
            "productId": "A1730",
            "productKey": "5c063821193f41e09f1c4fdd7567dda3",  # KR_PRODUCT_KEY in kr_sdk_config.json
            "projectId": "G153",
            "redirect_uri": 1,
            "response_type": "code",
            "sdkVersion": "2.6.0h",
            "channelId": "240",
        }
        data["sign"] = auth.encode_md5_parameter(
            data, constants.APP_KEYS[types.Game.WUWA][self.region]
        )

        if mmt_result is not None:
            data.update(mmt_result.get_game_dict())

        rsp = await self.request(routes.GAME_LOGIN.get_url(), data=data)

        if rsp["codes"] not in {0, None}:
            errors.raise_from_data(rsp)

        return models.GameLoginResult(**rsp)

    @decorators.region_specific(types.Region.OVERSEAS)
    async def get_game_token(
        self, code: str, *, device_id: str | None = None
    ) -> models.GameTokenResult:
        """Get game access token.

        ### Args:
            code: Authorization code.

        ### Returns:
            Game token result.
        """
        data: dict[str, typing.Any] = {
            "client_id": "7rxmydkibzzsf12om5asjnoo",  # KR_PRODUCT_KEY in kr_sdk_config.json
            "deviceNum": device_id or auth.generate_uuid_uppercase(),
            "client_secret": "32gh5r0p35ullmxrzzwk40ly",
            "code": code,
            "productId": "A1725",
            "projectId": "G153",
            "redirect_uri": 1,
            "grant_type": "authorization_code",
        }
        data["sign"] = auth.encode_md5_parameter(
            data, constants.APP_KEYS[types.Game.WUWA][self.region]
        )
        rsp = await self.request(routes.GAME_TOKEN.get_url(), data=data)

        if rsp["codes"] not in {0, None}:
            errors.raise_from_data(rsp)

        return models.GameTokenResult(**rsp)

    @decorators.region_specific(types.Region.OVERSEAS)
    async def check_game_token(self, access_token: str, *, device_id: str | None = None) -> int:
        """Check game access token life time.

        ### Args:
            access_token: Game access token.

        ### Returns:
            Remaining life time of access token in seconds.
        """
        data = {
            "deviceNum": device_id or auth.generate_uuid_uppercase(),
            "access_token": access_token,
            "productId": "A1725",
            "projectId": "G153",
            "channelId": "171",
        }
        data["sign"] = auth.encode_md5_parameter(
            data, constants.APP_KEYS[types.Game.WUWA][self.region]
        )
        rsp = await self.request(routes.GAME_TOKEN_CHECK.get_url(), data=data)

        if rsp["codes"] not in {0, None}:
            errors.raise_from_data(rsp)

        return rsp["expireSec"]

    @decorators.region_specific(types.Region.OVERSEAS)
    async def game_auto_login(
        self, auto_token: str, *, device_id: str | None = None
    ) -> models.GameLoginResult:
        """Login with an auto token.

        ### Args:
            auto_token: Auto login token.

        ### Returns:
            Login result.
        """
        data: dict[str, typing.Any] = {
            "token": auto_token,
            "client_id": "7rxmydkibzzsf12om5asjnoo",  # KR_PRODUCT_KEY in kr_sdk_config.json
            "deviceNum": device_id or auth.generate_uuid_uppercase(),
            "sdkVersion": "2.6.0h",
            "productId": "A1730",
            "projectId": "G153",
            "redirect_uri": 1,
            "response_type": "code",
            "channelId": "171",
        }
        data["sign"] = auth.encode_md5_parameter(
            data, constants.APP_KEYS[types.Game.WUWA][self.region]
        )
        rsp = await self.request(routes.GAME_AUTO_LOGIN.get_url(), data=data)

        if rsp["codes"] not in {0, None}:
            errors.raise_from_data(rsp)

        return models.GameLoginResult(**rsp)

    @decorators.region_specific(types.Region.OVERSEAS)
    async def generate_oauth_code(self, game_token: str, *, device_id: str | None = None) -> str:
        """Generate OAuth code from game token.

        ### Args:
            game_token: Game access token.

        ### Returns:
            The generated OAuth code.
        """
        data: dict[str, int | str] = {
            "client_id": "7rxmydkibzzsf12om5asjnoo",  # KR_PRODUCT_KEY in kr_sdk_config.json
            "deviceNum": device_id or auth.generate_uuid_uppercase(),
            "client_secret": "32gh5r0p35ullmxrzzwk40ly",
            "access_token": game_token,
            "productId": "A1725",
            "projectId": "G153",
            "redirect_uri": 1,
            "scope": "launcher",
        }
        rsp = await self.request(routes.OAUTH_CODE.get_url(), data=data)

        if rsp["codes"] not in {0, None}:
            errors.raise_from_data(rsp)

        return rsp["oauthCode"]



"""Types used in the library."""

import enum

__all__ = ["CaptchaId", "Game", "Lang"]


class Game(str, enum.Enum):
    """Kuro game."""

    PGR = "pgr"
    """Punishing: Gray Raven"""

    WUWA = "wuwa"
    """Wuthering Waves"""


class Lang(str, enum.Enum):
    """Languages supported by the API."""

    CHINESE_SIMPLIFIED = "zh-Hans"
    CHINESE_TRADITIONAL = "zh-Hant"
    ENGLISH = "en"
    JAPANESE = "ja"
    KOREAN = "ko"
    FRENCH = "fr"
    GERMAN = "de"
    SPANISH = "es"


class WuWaServer(str, enum.Enum):
    """WuWa server."""

    AMERICA = "591d6af3a3090d8ea00d8f86cf6d7501"
    """America"""

    ASIA = "86d52186155b148b5c138ceb41be9650"
    """Asia"""

    EUROPE = "6eb2a235b30d05efd77bedb5cf60999e"
    """Europe"""

    HMT = "919752ae5ea09c1ced910dd668a63ffb"
    """Hong Kong, Macau, Taiwan"""

    SEA = "10cd7254d57e58ae560b15d51e34b4c8"
    """Southeast Asia"""


class WuWaBanner(enum.IntEnum):
    """WuWa banner."""

    FEATURED_RESONATOR = 1
    """Featured Resonator Convene"""

    FEATURED_WEAPON = 2
    """Featured Weapon Convene"""

    STANDARD_RESONATOR = 3
    """"Standard Resonator Convene"""

    STANDARD_WEAPON = 4
    """Standard Weapon Convene"""

    BEGINNERS = 5
    """Beginner's Convene"""

    BEGINNERS_CHOICE = 6
    """Beginner's Choice Convene"""

    GIVEBACK_CUSTOM = 7
    """Beginner's Choice Convene (Giveback Custom Convene)"""


class Region(str, enum.Enum):
    """Region."""

    OVERSEAS = "os"
    """Overseas"""

    CHINESE = "cn"
    """Chinese"""


class CaptchaId(str, enum.Enum):
    """Captcha IDs."""

    KUROBBS = "ec4aa4174277d822d73f2442a165a2cd"
    """Kurobbs captcha Id"""

    GAME = "1f4565ff7acc97b1a2fc97b921743aa4"
    """Game captcha Id"""



"""Async API wrapper for KuroBBS written in Python.

Source Code: https://github.com/Wuthery/kuro.py
"""

from . import filters, models, utility
from .client import *
from .constants import *
from .errors import *
from .types import *

__version__ = "0.6.1"



"""Game auth models."""

import pydantic

from kuro.models import base

__all__ = ["GameLoginResult", "GameTokenResult"]


class GameLoginResult(base.APIModel):
    """Game login result."""

    username: str
    """Username."""
    sdk_user_id: str = pydantic.Field(alias="sdkuserid")
    """SDK user ID."""
    id: int
    """User ID."""
    login_type: int
    """Login type."""
    code: str
    """Code."""
    temp_token: str
    """Temporary token."""
    id_stat: int
    """ID status (?)."""
    user_type: int
    """User type."""
    cuid: str
    """CUID. Usually same as id but in string format."""
    show_paw: bool
    """Whether to show password (?)."""
    bind_dev_stat: bool
    """Bind device status."""
    bind_dev_switch: bool
    """Bind device switch."""
    auto_token: str
    """Auto token used to login automatically."""
    first_login: bool = pydantic.Field(alias="firstLgn")
    """Whether this is the first login."""
    email: str
    """Email."""
    bind: bool
    """Whether the account is bound (?)."""


class GameTokenResult(base.APIModel):
    """Game token result."""

    access_token: str
    """Access token."""
    expires_in: int
    """Expiration time."""



"""Web auth client."""

import typing

from kuro import errors, models
from kuro.client import routes
from kuro.client.components import base

__all__ = ["WebAuthClient"]


class WebAuthClient(base.BaseClient):
    """Web auth client."""

    @typing.overload
    async def send_sms_code(self, number: str, *, mmt_result: None = ...) -> bool: ...

    @typing.overload
    async def send_sms_code(
        self, number: str, *, mmt_result: models.MMTResult = ...
    ) -> typing.Literal[True]: ...

    async def send_sms_code(
        self, number: str, *, mmt_result: models.MMTResult | None = None
    ) -> bool:
        """Send SMS code to phone number.

        ### Args:
            number: Chinese phone number. With or without regional prefix (`+86`).
            mmt_result: MMT result with solved captcha.

        ### Returns:
            True if SMS code was sent successfully. False means geetest was triggered.
        """
        data = {
            "mobile": number if not number.startswith("+86") else number[3:],
            "geeTestData": mmt_result.model_dump_json() if mmt_result else "",
        }
        rsp = await self.request(routes.GET_SMS_CODE.get_url(), data=data, headers={"Source": "h5"})

        if not rsp["success"]:
            errors.raise_from_data(rsp)

        return not rsp["data"]["geeTest"]

    async def web_login(self, number: str, code: str) -> models.LoginResult:
        """Login with a phone number and OTP code.

        ### Args:
            number: Chinese phone number. With or without regional prefix (`+86`).
            code: OTP code.

        ### Returns:
            Login result.
        """
        data = {"mobile": number if not number.startswith("+86") else number[3:], "code": code}
        rsp = await self.request(routes.WEB_LOGIN.get_url(), data=data, headers={"Source": "h5"})

        if not rsp["success"]:
            errors.raise_from_data(rsp)

        return models.LoginResult(**rsp["data"])
